# -*- coding: utf-8 -*-

from __future__ import print_function, unicode_literals, absolute_import

import os
import sys
import re
from fileinput import input


class ThunderbirdNotFound(LookupError):
    pass


def absolute_path(config, thunderbird_base, section):
    if config.getint(section, 'IsRelative') == 1:
        return os.path.join(thunderbird_base, config.get(section, 'Path').replace('/', os.sep))
    else:
        return config.get(section, 'Path')


def profile_path(thunderbird_base):
    try:
        return sys.argv[1]
    except IndexError: pass

    try:
        from configparser import ConfigParser
    except:
        from ConfigParser import SafeConfigParser as ConfigParser

    config = ConfigParser()
    filename = os.path.join(thunderbird_base, 'profiles.ini')
    config.read(filename)
    profiles = {}
    for section in config.sections():
        name = str(section)
        if name[:7] == 'Install':
            try:
                profiles[config.get(section, 'Default').replace('/', os.sep)]
            except:
                profiles[config.get(section, 'Default').replace('/', os.sep)] = {'install': True}
            else:
                profiles[config.get(section, 'Default').replace('/', os.sep)]['install'] = True
        if name[:7] == 'Profile':
            try:
                profiles[config.get(section, 'Path').replace('/', os.sep)]
            except:
                profiles[config.get(section, 'Path').replace('/', os.sep)] = {'section': section}
            else:
                profiles[config.get(section, 'Path').replace('/', os.sep)]['section'] = section

    active_profile = None
    newest = 0
    for profile in profiles.values():
        try:
            if profile['install']:
                path = absolute_path(config, thunderbird_base, profile['section'])
                statinfo = os.stat(os.path.join(path, 'prefs.js'))
                mtime = statinfo.st_mtime
                if mtime > newest:
                    active_profile = profile
                    active_profile['Path'] = path
                    newest = mtime
        except KeyError: pass
        except OSError: pass

    if active_profile:
        return active_profile['Path']

    raise ThunderbirdNotFound


def prefs(thunderbird_base, key):
    c = re.compile(r'user_pref\(["\']' + key + '["\'],\s*["\']?(?P<value>[^"\']*)["\']?\);', re.UNICODE)
    filename = os.path.join(profile_path(thunderbird_base), 'prefs.js')
    try:
        with input(files=(filename,)) as finput:
            for line in finput:
                m = c.search(line)
                if m:
                    return m.group('value')
    except: pass
    return None


def enigmail_agentPath(thunderbird_base):
    return prefs(thunderbird_base, r'extensions\.enigmail\.agentPath')


def enigmail_juniorMode(thunderbird_base):
    result = prefs(thunderbird_base, r'extensions\.enigmail\.juniorMode')
    if result == '0' or result == None:
        return False
    else:
        return True

def enigmail_assignKeysByRules(thunderbird_base):
    result = prefs(thunderbird_base, r'extensions\.enigmail\.assignKeysByRules')
    if result == 'false':
        return False
    else:
        return True

def enigmail_assignKeysByEmailAddr(thunderbird_base):
    result = prefs(thunderbird_base, r'extensions\.enigmail\.assignKeysByEmailAddr')
    if result == 'false':
        return False
    else:
        return True

# This function reads through prefs.js and returns a dictionary:
# For example:
# user_pref("mail.identity.id10.fullName", "Dr Dog");
# user_pref("mail.identity.id10.useremail", "dog@dog.com");
# is returned as
# {
#   id10: { fullName: "Dr Dog", useremail: "dog@dog.com" }
# }
def identities(thunderbird_base):
    result = {}
    c = re.compile(r'user_pref\(["\']mail\.identity\.(?P<name>\w+)\.(?P<field>\w+)["\'],\s*["\'](?P<value>[^"\']*)["\']\);', re.UNICODE)
    filename = os.path.join(profile_path(thunderbird_base), 'prefs.js')
    for line in input(files=(filename,)):
        m = c.search(line)
        if m:
            if not m.group('name') in result:
                result[m.group('name')] = {}
            result[m.group('name')][m.group('field')] = m.group('value')
    return result

