#!/bin/sh

set -e

user_inst=0

export TEXTDOMAIN="tb_setup"
export TEXTDOMAINDIR="${SETUP_DIR}/po"
tr () {
    command gettext tb_setup "$@" 2>/dev/null || echo "$@"
}

while getopts hub:P:O:X:V:Z: opt ; do
    case $opt in
    \?|h) echo $(tr "Usage:") " $0: [-u] [-b <setup_base>]" ; exit 0 ;;
    u)    user_inst=1 ;;
    b)    setup_base=$OPTARG/ ;;
    esac
done
shift $(($OPTIND - 1))
SETUP_DIR=${setup_base:=.}
[ ! -d "${setup_base}/.Setup" ] || SETUP_DIR=${setup_base}/.Setup

_section () {
    echo "*** $@ ***" >&2
}

_info () {
    echo "--- $@" >&2
}

_warn () {
    echo "!!! $@" >&2
}

_install () {
    mode=$1
    owners=$2
    src=$3
    dst=$4
    _info $(tr "Installing:") "$dst"
    [ ! -e "$dst" ] || chmod 0600 "$dst"
    mkdir -p "$(dirname "$dst")"
    cp -f "$src" "$dst"
    [ -z "$owners" ] || chown "$owners" "$dst"
    chmod "$mode" "$dst"
}

_start_adapter () {
    _info $(tr "Starting the adapter")
    if [ ! -e "$HOME/.config/systemd/user/default.target.wants/pEp-mini-json-adapter.service" ] ; then
        "$adapter_bin" --no-html &
    fi
}

adapter_bin=""
if [ $user_inst -eq 0 ] ; then
    _warn $(tr "Not supported in this version of the installer.")
    exit 1
else
    _section $(tr "Install adapter to user profile")
    if [ ! -s "${SETUP_DIR}/pEp-mini-json-adapter.user" ] ; then
        _warn $(tr "This installer does not contain the user variant of the adapter")
        exit 1
    fi
    share_prefix=$HOME/.local/share
    bin_prefix=$HOME/.local/bin
    etc_xdg_prefix=${XDG_CONFIG_HOME:-$HOME/.config}
    umask 0027
    adapter_bin="${bin_prefix}/pEp-mini-json-adapter"
    _install 0555 "" \
        "${SETUP_DIR}/pEp-mini-json-adapter.user" \
        "$adapter_bin"
    _install 0555 "" \
        "${SETUP_DIR}/pEp-mini-json-adapter-wrapper" \
        "${bin_prefix}/pEp-mini-json-adapter-wrapper"
    _install 0555 "" \
        "${SETUP_DIR}/pEp-mini-json-adapter-cleanup" \
        "${bin_prefix}/pEp-mini-json-adapter-cleanup"
    _install 0444 "" \
        "${SETUP_DIR}/system.db" \
        "${share_prefix}/pEp/system.db"

    _install 0555 "" \
        "${SETUP_DIR}/pEp-update-service" \
        "${bin_prefix}/pEp-update-service"
    _install 0555 "" \
        "${SETUP_DIR}/pEp-install.sh" \
        "${bin_prefix}/pEp-install.sh"
    _install 0444 "" \
        "${SETUP_DIR}/locale/de/LC_MESSAGES/pEp-update-service.mo" \
        "${share_prefix}/locale/de/LC_MESSAGES/pEp-update-service.mo"
    _install 0444 "" \
        "${SETUP_DIR}/locale/en/LC_MESSAGES/pEp-update-service.mo" \
        "${share_prefix}/locale/en/LC_MESSAGES/pEp-update-service.mo"
    _install 0444 "" \
        "${SETUP_DIR}/pep.png" \
        "${share_prefix}/icons/pEp/pep.png"
    _install 0444 "" \
        "${SETUP_DIR}/pep-attention.png" \
        "${share_prefix}/icons/pEp/pep-attention.png"
    _install 0444 "" \
        "${SETUP_DIR}/product-tblinux.ini" \
        "$HOME/.pEp/products/product-tblinux.ini"
    _install 0444 "" \
        "${SETUP_DIR}/product-tblinux.der" \
        "$HOME/.pEp/products/product-tblinux.der"

    if [ ! -f "$HOME/.pEp/updater.ini" ]; then
        _install 0644 "" \
            "${SETUP_DIR}/updater.ini" \
            "$HOME/.pEp/updater.ini"
        sed -i "s|locale_dir=.*|locale_dir=$share_prefix/locale/|g" "$HOME/.pEp/updater.ini"
        sed -i "s|.*normal_icon=.*|normal_icon=${share_prefix}/icons/pEp/pep.png|g" "$HOME/.pEp/updater.ini"
        sed -i "s|.*attention_icon=.*|attention_icon=${share_prefix}/icons/pEp/pep-attention.png|g" "$HOME/.pEp/updater.ini"
    fi

    systemctl --user daemon-reload > /dev/null 2>&1 && use_systemd=yes || use_systemd=

    if [ -n "$use_systemd" ] ; then
        _install 0444 "" \
            "${SETUP_DIR}/pEp-mini-json-adapter.service.user" \
            "${etc_xdg_prefix}/systemd/user/pEp-mini-json-adapter.service"
        _install 0444 "" \
            "${SETUP_DIR}/pEp-update-service.service.user" \
            "${etc_xdg_prefix}/systemd/user/pEp-update-service.service"
    else
        mkdir -p "${etc_xdg_prefix}/autostart"
        _install 0444 "" \
            "${SETUP_DIR}/pEp-mini-json-adapter.desktop.user" \
            "${etc_xdg_prefix}/autostart/pEp-mini-json-adapter.desktop"
        _install 0444 "" \
            "${SETUP_DIR}/pEp-update-service.desktop.user" \
            "${etc_xdg_prefix}/autostart/pEp-update-service.desktop"
    fi
    _section $(tr "Configure user-local adapter for user")
    _info $(tr "Stopping adapters...")
    pkill pEp-mini-json-adapter || true
    _info $(tr "Starting adapters...")
    if [ -f "${etc_xdg_prefix}/systemd/user/pEp-mini-json-adapter.service" ] ; then
        if [ -n "$use_systemd" ] ; then
          _info $(tr "Activating systemd unit...")
          systemctl --user daemon-reload
          systemctl --user stop pEp-mini-json-adapter.service 2>/dev/null || true
          systemctl --user enable pEp-mini-json-adapter.service
          systemctl --user start pEp-mini-json-adapter.service

          systemctl --user stop pEp-update-service.service 2>/dev/null || true
          systemctl --user enable pEp-update-service.service
          systemctl --user start pEp-update-service.service
        fi
    fi
    _start_adapter
fi
_info $(tr "Completed")
